local Player = {}
local Collision = require "collision"


--require "tools/camera"
local rect = require "objects/rect"

local floor = math.floor


function Player:new(x,y,world)

	--Initialize player with x,y,w,h,id 
	local player = require("objects/entity"):new(x,y,64,64,"player")
	player.vel.x = 150
	player.vel.y = 0
	player.gravity = 500
	player.world = world
	player.isJumping = false
	player.isFalling = false
	player.previousPos = {}
	
	--function to load the player
	function player:load()
		gameLoop:addLoop(self)
		renderer:addRenderer(self)
	end

	local key = love.keyboard.isDown

	--function to update the player
	function player:tick(dt)
	
		--this part here is for updating the list of previous positions. It's not yet used
		local previousX = self.pos.x
		local previousY = self.pos.y	

		table.insert(player.previousPos, {previousX,previousY})
		if #player.previousPos > 3 then
			table.remove(player.previousPos, 1)
		end
		--end of the previous position part
	
		function round(x) return x + 0.5 - (x + 0.5) % 1 end
		

		
		--movement based on the keys
		if key("q") then
			self.dir.x = -1
			self.pos.x = self.pos.x + (self.vel.x * dt) * self.dir.x
			self.pos.x = round(self.pos.x)
			if Collision.withGroundLeft(self.pos.x, self.pos.y, self.size.x, self.world) then
				self.pos.x = previousX			
			end
		end

		if key("d") then
			self.dir.x = 1
			self.pos.x = self.pos.x + (self.vel.x * dt) * self.dir.x
			self.pos.x = round(self.pos.x)
			if Collision.withGroundRight(self.pos.x, self.pos.y, self.size.x, self.world) then
				self.pos.x = previousX
			end
		end
		
		if key("z") and (player.isFalling == false and player.isJumping == false) then
			player.isJumping = true
			self.vel.y = -350
			if Collision.withGroundUp(self.pos.x, self.pos.y, self.size.x, self.world) then
				--self.pos.y = previousY
			end
		end
		
		
		--two moments to update speed and position: while falling (gravity) and while jumping
		if not Collision.withGround(self.pos.x, self.pos.y, self.size.x, self.world) or (player.isJumping == true and player.isFalling == false)  then
			self.pos.y = self.pos.y + (self.vel.y * dt) + ((self.gravity * dt * dt) / 2)
			self.vel.y = self.vel.y + (self.gravity * dt)
			self.pos.y = round(self.pos.y)
			if (self.vel.y >= 0) then
				player.isFalling = true
				player.isJumping = false
			end
			
			--if the player hit a ceiling then set velocity to 0 and start falling
			if Collision.withGroundUp(self.pos.x, self.pos.y, self.size.x, self.world) then
				player.isFalling = true
				player.isJumping = false
				self.pos.y = previousY
				self.vel.y = 0
			end

		else
		--if the speed is positive and the player is touching ground then set speed to 0
			if self.vel.y > 0 then
				self.vel.y = 0
				player.isFalling = false
				self.pos.y = previousY 
			end
		end
		
	
	end

	--draw the player
	function player:draw()
		love.graphics.setColor(255,0,0)
		love.graphics.rectangle("fill",self.pos.x,self.pos.y,self.size.x,self.size.y)
	end
	
	--unused function to return a valid previous position
	function player:positionFix()
		calculatedPosX = nil
		calculatedPosY = nil
		lastPos = 1
		for position = 1,#player.previousPos do
			lastPos = lastPos + 1
			if not Collision.withGroundUp(player.previousPos[position][1], player.previousPos[position][2], player.size.x, player.world) then
				calculatedPosX = player.previousPos[position][1]
				calculatedPosY = player.previousPos[position][2]
			end
		end
		
		if calculatedPosX == nil then
			calculatedPosX = player.previousPos[lastPos][1]
			calculatedPosY = player.previousPos[lastPos][2]
		end
		
		return {calculatedPosX, calculatedPosY}
	end
	

	return player
end

return Player